/*
 Wireless Game Controller - SNES Drop in TX
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 14/03/2019
 Last Modified: 01/06/2019
 
 Use your SNES as a wireless game controller on any of our WGC receivers.
  
 Program the ATtiny48/88 to use an internal 8MHz oscillator and divide clock by 1
 avrdude -p ATtiny48 -c usbasp -U lfuse:w:0xEE:m -U hfuse:w:0xDF:m -U efuse:w:0xFF:m
 
*/

// ATtiny48/88 Pin map
// -------------------------------------
// PB0 - LED			PC0 - Up			PD0 - B
// PB1 - nRF CE		PC1 - Left		PD1 - Y
// PB2 - nRF CSN		PC2 - Down		PD2 - A
// PB3 - nRF MOSI		PC3 - Right		PD3 - X
// PB4 - nRF MISO		PC4 - Select	PD4 - R					
// PB5 - nRF SCK		PC5 - Start		PD6 - LED
// 						PC7 - L			
//	

#define F_CPU 8000000 // 8 MHz

#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include "setup.c"
#include "mirf.c"

uint32_t timeOutCounter = 0;
uint16_t startButtonCounter = 0;
uint8_t inPowerDown = 1;


void configure_nrf(void) {
	uint8_t inConfig = 1;
	nrfChannel = 1;
	nrfTx[4] = 1;
	nrfPpr = 1;
	
	while (inConfig == 1) {
		PORTD |= (1<<LED);
		
		if (!(PINC & (1<<BUTTON_UP))) { // Up, channel increase
			PORTD &= ~(1<<LED);
			if (nrfChannel < 127) {
				nrfChannel++;
			}
			while (!(PINC & (1<<BUTTON_UP))) { // Wait until let go
				_delay_ms(20);
			}
		}
		if (!(PINC & (1<<BUTTON_DOWN))) {  // Down, channel decrease
			PORTD &= ~(1<<LED);
			if (nrfChannel > 1) {
				nrfChannel--;
			}
			while (!(PINC & (1<<BUTTON_DOWN))) {
				_delay_ms(20);
			}
		}
		if (!(PINC & (1<<BUTTON_LEFT))) { // Left, address decrease
			PORTD &= ~(1<<LED);
			if (nrfTx[4] > 1) {
				nrfTx[4]--;
			}
			while (!(PINC & (1<<BUTTON_LEFT))) {
				_delay_ms(20);
			}
		}
		if (!(PINC & (1<<BUTTON_RIGHT))) { // Right, address increase
			PORTD &= ~(1<<LED);
			if (nrfTx[4] < 255) {
				nrfTx[4]++;
			}
			while (!(PINC & (1<<BUTTON_RIGHT))) {
				_delay_ms(20);
			}
		}
		if (!(PINC & (1<<BUTTON_L))) { // Select, PPK decrease
			PORTD &= ~(1<<LED);
			if (nrfPpr > 1) {
				nrfPpr--;
			}
			while (!(PINC & (1<<BUTTON_SELECT))) {
				_delay_ms(20);
			}
		}
		if (!(PIND & (1<<BUTTON_R))) { // Start,  PPK increase
			PORTD &= ~(1<<LED);
			if (nrfPpr < 4) {
				nrfPpr++;
			}
			while (!(PINC & (1<<BUTTON_START))) {
				_delay_ms(20);
			}
		}
		if (!(PIND & (1<<BUTTON_A))) { // A, finish and transmit
			PORTD &= ~(1<<LED);
			while (!(PIND & (1<<BUTTON_A))) {
				_delay_ms(20);
			}
			
			_delay_ms(500);
			inConfig = 0;
		}
	}					
	
	// Update the EEPROM
	eeprom_write_byte((uint8_t*) nrfChannelLocation, nrfChannel);
	eeprom_write_byte((uint8_t*) nrfTxLocation, nrfTx[4]);
	eeprom_write_byte((uint8_t*) nrfPprLocation, nrfPpr);
	
	// Update the nRF settings to transmit the data to the RX
	mirf_config_setting_update();
	
	// Transmit the nrf setting to the receiver until powered off (no need to transmit PPK)
	NRF_TX_POWERUP; // Power up to transmitter mode
	_delay_ms(5);
	mirf_flush_rx_tx();
	
	kpData[0] = nrfChannel;
	kpData[1] = nrfTx[4];
	while (1) {
		PORTD |= (1<<LED);
		mirf_transmit_data(2);
		_delay_ms(50);
		PORTD &= ~(1<<LED);
		_delay_ms(50);
		
		if (!(PIND & (1<<BUTTON_A))) { // A, go back to sleep
			while (!(PIND & (1<<BUTTON_A))) {
				_delay_ms(50);
			}
			
			// Set nRF settings back
			mirf_config();
			
			break;
		}
	}
}


int main (void) {
	setup();
	
	while(1) {
		// Power down nRF24 and MCU
		if (inPowerDown == 1) {
			PORTD &= ~(1<<LED);
			NRF_POWERDOWN;
			PCICR |= (1<<PCIE1); // Enable interrupt to wake us up
			system_sleep();
			
			PCICR &= ~(1<<PCIE1); // Disable interrupt
			timeOutCounter = 0;
			startButtonCounter = 0;
			
			// Select button pressed, check if held for 2 seconds, then configure nRF
			if (!(PINC & (1<<BUTTON_SELECT))) { // Select button pressed
				uint16_t selectCounter = 0;
				while (!(PINC & (1<<BUTTON_SELECT))) {
					_delay_ms(50);
					selectCounter++;
				}
				
				// Held for 2 seconds?
				if (selectCounter >= 40) {
					while (!(PINC & (1<<BUTTON_SELECT))) { // Wait until let go
						_delay_ms(50);
					}
					configure_nrf();
				}
			}
			else if (!(PINC & (1<<BUTTON_START))) { 
				NRF_TX_POWERUP; // Power up to transmitter mode
				_delay_ms(5);
				inPowerDown = 0;
				PORTD |= (1<<LED);
			}
		}
		else {
			// Clear existing data
			kpData[1] = 0;
			kpData[2] = 0;
			
			// Check buttons
			if (!(PINC & (1<<BUTTON_UP))) { // Up
				kpData[1] |= 0x08;
			}
			if (!(PINC & (1<<BUTTON_DOWN))) { // Down
				kpData[1] |= 0x04;
			}
			if (!(PINC & (1<<BUTTON_LEFT))) { // Left
				kpData[1] |= 0x02;
			}
			if (!(PINC & (1<<BUTTON_RIGHT))) { // Right
				kpData[1] |= 0x01;
			}
			if (!(PINC & (1<<BUTTON_SELECT))) { // Select
				kpData[1] |= 0x20;
			}
			if (!(PINC & (1<<BUTTON_START))) { // Start
				kpData[1] |= 0x10;
			}
			if (!(PIND & (1<<BUTTON_B))) { // B
				kpData[1] |= 0x80;
			}
			if (!(PIND & (1<<BUTTON_A))) { // A
				kpData[2] |= 0x80;
			}
			if (!(PIND & (1<<BUTTON_Y))) { // Y
				kpData[1] |= 0x40;
			}
			if (!(PIND & (1<<BUTTON_X))) { // X
				kpData[2] |= 0x40;
			}
			if (!(PINC & (1<<BUTTON_L))) { // L
				kpData[2] |= 0x20;
			}
			if (!(PIND & (1<<BUTTON_R))) { // R
				kpData[2] |= 0x10;
			}
			
			// Transmit
			kpData[0] = SNES_TX;
			for (uint8_t t = 0; t < nrfPpr; t++) {
				mirf_transmit_data(3);
			}
			_delay_ms(2);
			
			// Set back to power down after ~5 minutes of no activity
			if (kpData[0] == 0) { // No keys pressed, increment timeout counter
				timeOutCounter++;
				
				if (timeOutCounter >= 150000) {
					inPowerDown = 1;
					timeOutCounter = 0;
				}
			}
			else {
				timeOutCounter = 0;
			}
			
			// Start button held when powered on, wait 5 seconds and then power down
			if (!(PINC & (1<<BUTTON_START))) {
				startButtonCounter++;
				
				if (startButtonCounter >= 1600) {
					inPowerDown = 1;
					startButtonCounter = 0;
					PORTD &= ~(1<<LED);
					
					while (!(PINC & (1<<BUTTON_START))) {
						_delay_ms(20);
					}
					_delay_ms(200);
				}
			}
			else {
				startButtonCounter = 0;
			}
		}
	}
	
	return 0;
}

EMPTY_INTERRUPT(WDT_vect);
EMPTY_INTERRUPT(PCINT1_vect);
